var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;
exports.replaceNode = replaceNode;
exports.createStringLiteral = createStringLiteral;
exports.createTempVarIdentifier = createTempVarIdentifier;
exports.createAssignmentExprStmt = createAssignmentExprStmt;
exports.createVarDeclaration = createVarDeclaration;
exports.createProcessScriptMethCall = createProcessScriptMethCall;
exports.createLocationGetWrapper = createLocationGetWrapper;
exports.createLocationSetWrapper = createLocationSetWrapper;
exports.createPropertySetWrapper = createPropertySetWrapper;
exports.createMethCallWrapper = createMethCallWrapper;
exports.createPropertyGetWrapper = createPropertyGetWrapper;
exports.createComputedPropertyGetWrapper = createComputedPropertyGetWrapper;
exports.createComputedPropertySetWrapper = createComputedPropertySetWrapper;
exports.createExpandedConcatOperation = createExpandedConcatOperation;
// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------

var _parsingTools = require('./parsing-tools');

var _internalLiteral = require('./internal-literal');

var _internalLiteral2 = _interopRequireDefault(_internalLiteral);

var _instruction = require('./instruction');

var _instruction2 = _interopRequireDefault(_instruction);

function replaceNode(node, newNode, parent, key) {
    if (key === 'arguments' || key === 'elements' || key === 'expressions') {
        var idx = parent[key].indexOf(node);

        parent[key][idx] = newNode;
    } else parent[key] = newNode;
}

function createStringLiteral(value) {
    return {
        type: _parsingTools.Syntax.Literal,
        value: value,
        raw: '"' + value + '"'
    };
}

function createTempVarIdentifier() {
    return {
        type: _parsingTools.Syntax.Identifier,
        name: _internalLiteral2.default.tempVar
    };
}

function createAssignmentExprStmt(left, right) {
    return {
        type: _parsingTools.Syntax.ExpressionStatement,

        expression: {
            type: _parsingTools.Syntax.AssignmentExpression,
            operator: '=',
            left: left,
            right: right
        }
    };
}

function createVarDeclaration(identifier) {
    return {
        type: _parsingTools.Syntax.VariableDeclaration,

        declarations: [{
            type: _parsingTools.Syntax.VariableDeclarator,
            id: identifier,
            init: null
        }],

        kind: 'var'
    };
}

function createProcessScriptMethCall(arg, isApply) {
    var ast = {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: _instruction2.default.processScript
        },

        arguments: [arg]
    };

    if (isApply) {
        ast.arguments.push({
            type: _parsingTools.Syntax.Literal,
            value: true,
            raw: 'true'
        });
    }

    return ast;
}

function createLocationGetWrapper() {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: _instruction2.default.getLocation
        },

        arguments: [{
            type: _parsingTools.Syntax.Identifier,
            name: 'location'
        }]
    };
}

function createLocationSetWrapper(value) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.MemberExpression,
            computed: false,

            object: {
                type: _parsingTools.Syntax.FunctionExpression,
                id: null,
                params: [],
                defaults: [],

                body: {
                    type: _parsingTools.Syntax.BlockStatement,
                    body: [{
                        type: _parsingTools.Syntax.ReturnStatement,

                        argument: {
                            type: _parsingTools.Syntax.LogicalExpression,
                            operator: '||',

                            left: {
                                type: _parsingTools.Syntax.CallExpression,

                                callee: {
                                    type: _parsingTools.Syntax.Identifier,
                                    name: _instruction2.default.setLocation
                                },

                                arguments: [{
                                    type: _parsingTools.Syntax.Identifier,
                                    name: 'location'
                                }, value]
                            },

                            right: {
                                type: _parsingTools.Syntax.AssignmentExpression,
                                operator: '=',

                                left: {
                                    type: _parsingTools.Syntax.Identifier,
                                    name: 'location'
                                },

                                right: value
                            }
                        }
                    }]
                },

                rest: null,
                generator: false,
                expression: false
            },

            property: {
                type: _parsingTools.Syntax.Identifier,
                name: 'apply'
            }
        },

        arguments: [{
            type: _parsingTools.Syntax.ThisExpression
        }]
    };
}

function createPropertySetWrapper(propertyName, obj, value) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: _instruction2.default.setProperty
        },

        arguments: [obj, createStringLiteral(propertyName), value]
    };
}

function createMethCallWrapper(owner, meth, args) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: _instruction2.default.callMethod
        },

        arguments: [owner, meth, {
            type: _parsingTools.Syntax.ArrayExpression,
            elements: args
        }]
    };
}

function createPropertyGetWrapper(propertyName, owner) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: _instruction2.default.getProperty
        },

        arguments: [owner, createStringLiteral(propertyName)]
    };
}

function createComputedPropertyGetWrapper(property, owner) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: _instruction2.default.getProperty
        },

        arguments: [owner, property]
    };
}

function createComputedPropertySetWrapper(property, owner, value) {
    return {
        type: _parsingTools.Syntax.CallExpression,

        callee: {
            type: _parsingTools.Syntax.Identifier,
            name: _instruction2.default.setProperty
        },

        arguments: [owner, property, value]
    };
}

function createExpandedConcatOperation(left, right) {
    return {
        type: _parsingTools.Syntax.AssignmentExpression,
        operator: '=',
        left: left,

        right: {
            type: _parsingTools.Syntax.BinaryExpression,
            operator: '+',
            left: left,
            right: right
        }
    };
}