var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;
// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------

var _transform = require('./transform');

var _transform2 = _interopRequireDefault(_transform);

var _instruction = require('./instruction');

var _instruction2 = _interopRequireDefault(_instruction);

var _parsingTools = require('./parsing-tools');

// Const
var HTML_COMMENT_RE = /(^|\n)\s*<!--[.\r]*(\n|$)/g;
var ARRAY_RE = /^\s*\[[\s\S]*\]\s*$/;
var OBJECT_RE = /^\s*\{[\s\S]*\}\s*$/;

var PROCESSED_SCRIPT_RE = new RegExp([_instruction2.default.getLocation, _instruction2.default.setLocation, _instruction2.default.getProperty, _instruction2.default.setProperty, _instruction2.default.callMethod, _instruction2.default.processScript].join('|').replace(/\$/, '\\$'));

// Utils
function replaceHtmlComments(code) {
    do code = code.replace(HTML_COMMENT_RE, '\n'); while (HTML_COMMENT_RE.test(code));

    return code;
}

function isArrayCode(code) {
    return ARRAY_RE.test(code);
}

function isObjectCode(code) {
    return OBJECT_RE.test(code);
}

function isJSONCode(code) {
    if (isObjectCode(code)) {
        try {
            JSON.parse(code);

            return true;
        } catch (e) {
            return false;
        }
    }

    return false;
}

exports.default = {
    isScriptProcessed: function (code) {
        return PROCESSED_SCRIPT_RE.test(code);
    },

    isDataScript: function (code) {
        return isObjectCode(code) || isArrayCode(code);
    },

    process: function (code, beautify) {
        var isJSON = isJSONCode(code);
        var isObject = isObjectCode(code);

        // NOTE: The JS parser removes the line that follows'<!--'. (T226589)
        var result = replaceHtmlComments(code);
        var ast = null;

        try {
            ast = _parsingTools.parse(isObject && !isJSON ? '(' + result + ')' : 'function temp(){\n' + result + '\n}');
        } catch (e) {
            try {
                if (isObject && !isJSON) {
                    ast = _parsingTools.parse('function temp(){\n' + result + '\n}');
                    isObject = false;
                } else return code;
            } catch (err) {
                return code;
            }
        }

        if (!_transform2.default(ast)) return code;

        result = _parsingTools.generate(ast, {
            format: {
                quotes: 'double',
                escapeless: true,
                compact: !beautify
            },

            json: isJSON
        });

        if (isObject && !isJSON) result = result.replace(/^\(|\);\s*$/g, '');else result = result.replace(/^\s*function\s+temp\s*\(\s*\)\s*{\s*/, '').replace(/\s*}\s*$/, '');

        if (!/;\s*$/.test(code)) result = result.replace(/;\s*$/, '');

        return result;
    }
};
module.exports = exports.default;