exports.__esModule = true;
// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------

var _ast = require('../ast');

var _parsingTools = require('../parsing-tools');

var _instrumented = require('../instrumented');

// Transform:
// obj.<wrappable-property> -->
// __get$(obj, '<wrappable-property>')

exports.default = {
    nodeReplacementRequireTransform: true,

    nodeTypes: [_parsingTools.Syntax.MemberExpression],

    condition: function (node, parent) {
        if (node.computed) return false;

        if (!_instrumented.shouldInstrumentProperty(node.property.name)) return false;

        // Skip: object.prop = value
        if (parent.type === _parsingTools.Syntax.AssignmentExpression && parent.left === node) return false;

        // Skip: delete object.prop
        if (parent.type === _parsingTools.Syntax.UnaryExpression && parent.operator === 'delete') return false;

        // Skip: object.prop()
        if (parent.type === _parsingTools.Syntax.CallExpression && parent.callee === node) return false;

        // Skip: object.prop++ || object.prop-- || ++object.prop || --object.prop
        if (parent.type === _parsingTools.Syntax.UpdateExpression && parent.operator === '++' || parent.operator === '--') return false;

        // Skip: new (object.prop)() || new (object.prop)
        if (parent.type === _parsingTools.Syntax.NewExpression && parent.callee === node) return false;

        // Skip: for(object.prop in source)
        if (parent.type === _parsingTools.Syntax.ForInStatement && parent.left === node) return false;

        return true;
    },

    run: function (node, parent, key) {
        var newNode = _ast.createPropertyGetWrapper(node.property.name, node.object);

        _ast.replaceNode(node, newNode, parent, key);
    }
};
module.exports = exports.default;